/*
 * GCD -- compute the greatest common divisor of two integers recursively
 *
 * Usage: gcd n m
 *
 * Inputs: none
 * Outputs: gcd(m, n) = g	-- m, n as above; g is gcd(m, n)
 * Errors:	- other than 2 arguments supplied (not counting program)
 * 		- m, n not an integer
 *
 * Assumptions:	- the arguments are not digits followed by non-digits
 * 		  (unchecked)
 * 		- at least 1 of m, n is non-zero (checked)
 *
 * Written by Matt Bishop for ECS 30
 * version 1	November 2, 2015
 *
 *
 */
#include <stdio.h>

/*
 * GCD -- compute the GCD of m and n recursively
 * Parameters: m, n	2 integers of which at least 1 is non-zero
 * Returns: gcd of m and n
 * Assumptions: - at least 1 of m, n is non-zero (not checked)
 * 		- neither m, n are negative (not checked)
 * Algorithm: Euclid's Algorithm, using the relation:
 *	gcd(m, n) = gcd(n, n - m)
 * 
 */
int gcd(int m, int n)
{
	/* base case */
	if (n == 0) return(m);

	/* recursive case */
	return(gcd(n, m % n));
}


/*
 * the main routine
 */
int main(int argc, char *argv[])
{
	int m, n;		/* numbers to find the GCD of */

	/*
	 * be sure there are 2 arguments and the program name
	 */
	if (argc != 3){
		fprintf(stderr, "Usage: %s integer integer\n", argv[0]);
		return(1);
	}

	/*
	 * the two arguments are to be integers
	 * here, we assume there is a leading integer
	 */
	if (sscanf(argv[1], "%d", &m) != 1){
		fprintf(stderr, "%s: not an integer\n", argv[1]);
		return(1);
	}
	if (sscanf(argv[2], "%d", &n) != 1){
		fprintf(stderr, "%s: not an integer\n", argv[2]);
		return(1);
	}

	/*
	 * handle error case: if both m, n are 0, the gcd
	 * is undefined
	 */
	if (m == 0 && n == 0){
		fprintf(stderr, "%s: need at least 1 non-zero argument\n", argv[0]);
		return(1);
	}

	/*
	 * handle negative numbers
	 * gcd of two negative numbers or a negative and a positive number
	 * is the same as the gcd of their absolute value
	 */
	if (n < 0) n = -n;
	if (m < 0) m = -m;

	/*
	 * all's right with the world
	 * compute and print the GCD
	 */
	printf("gcd(%d, %d) = %d\n", m, n, gcd(m, n));

	/*
	 * the world ends (well, this program does ...)
	 */
	return(0);
}
